#include <DFPlayerMini_Fast.h>

DFPlayerMini_Fast myPlayer;

#define AILERON_OFFSET 0
#define ELEVATOR_OFFSET 0
#define RUDDER_OFFSET 28

#define THROTTLE_MAX 165
#define AILERON_MAX 340
#define ELEVATOR_MAX 390
#define RUDDER_MAX 335

#define THROTTLE_MIN 50
#define AILERON_MIN 274
#define ELEVATOR_MIN 224
#define RUDDER_MIN 234

#define AILERON_MAX_LOWRATES 325
#define ELEVATOR_MAX_LOWRATES 350
#define RUDDER_MAX_LOWRATES 292

#define AILERON_MIN_LOWRATES 289
#define ELEVATOR_MIN_LOWRATES 264
#define RUDDER_MIN_LOWRATES 264

#define THROTTLE_MIN_ADC 36960
#define AILERON_MIN_ADC 36300
#define ELEVATOR_MIN_ADC 37550
#define RUDDER_MIN_ADC 38380

#define THROTTLE_MAX_ADC 62800
#define AILERON_MAX_ADC 61700
#define ELEVATOR_MAX_ADC 63710
#define RUDDER_MAX_ADC 63800

/////////////////////////////////////////////////////////////////////////////////////
//analog pin assignments
const byte rollPin = 0;
const byte pitchPin = 1;
const byte throttlePin = 2;
const byte yawPin = 3;
/////////////////////////////////////////////////////////////////////////////////////

//loop total delay in ms
const byte delayTime = 0;

/////////////////////////////////////////////////////////////////////////////////////
//variables for control surface values
unsigned int pitchValue = 0;
unsigned int rollValue = 0;
unsigned int yawValue = 0;
unsigned int throttleValue = 0;
/////////////////////////////////////////////////////////////////////////////////////



void setup()
{
  pinMode(13, OUTPUT);
  digitalWrite(13, HIGH);

  pinMode(0, INPUT_PULLUP); //pitch rates
  pinMode(1, INPUT_PULLUP); //roll rates
  pinMode(2, INPUT_PULLUP); //rudder rates
  
  analogReadResolution(16);
  
  Serial.begin(115200);
  Serial4.begin(9600);
  Serial5.begin(9600);
  myPlayer.begin(Serial4);

  myPlayer.volume(30);
  delay(10);
  myPlayer.play(1);
}



void loop()
{
  //DAQ from joystick/switches
  getData();

  //send data to plane IFC
  sendData();

  //wait for IFC to "catch up"
  delay(delayTime);
}



void getData()
{
  //read and map joystick data
  //mapping normalized to max and min joystick values

  gitThrottle();
  gitRoll();
  gitPitch();
  gitYaw();

  return;
}



//////////////////////////////////////////////////////
// determines if data needs to be sent to plane and
// sends data to the plane using Serial2
//////////////////////////////////////////////////////
void sendData()
{
  Serial5.write(200);       
  constructAndSendMessages(0); //throttleValue
  constructAndSendMessages(1); //pitchValue
  constructAndSendMessages(2); //rollValue
  constructAndSendMessages(3); //yawValue
  Serial5.write(100);

  return;
}



//////////////////////////////////////////////////////
// determines message ID, compiles message, and
// sends complete message to the plane using Serial2
//////////////////////////////////////////////////////
void constructAndSendMessages(byte messageID)
{
  byte message1 = 0;
  byte message2 = 0;
  byte message3 = 0;

  message1 = messageID;

  if (messageID == 0) //----------------------------throttle
  {
    message1 = 4;
    message2 = 0;
    message3 = throttleValue & 0xFF;
    //Serial.println(message2);
  }
  else if (messageID == 1) //-----------------------pitch
  {
    message2 = (pitchValue >> 8) & 0xFF;
    message3 = pitchValue & 0xFF;
    //Serial.println(message2);
  }
  else if (messageID == 2) //-----------------------roll
  {
    message2 = (rollValue >> 8) & 0xFF;
    message3 = rollValue & 0xFF;
    //Serial.println(message2);
  }
  else if (messageID == 3) //-----------------------yaw
  {
    message2 = (rollValue >> 8) & 0xFF;
    message3 = yawValue & 0xFF;
    //Serial.println(message2);
  }
  else
  {
    //do nothing
  }

  //send data (messageID first and then raw data)
  Serial5.write(message1);
  Serial5.write(message2);
  Serial5.write(message3);

  //Serial.print(message1); Serial.print(" ");
  //Serial.println(message2);

  return;
}



void gitPitch()
{
  pitchValue = analogRead(pitchPin);
  if(digitalRead(0))
  {
    pitchValue = map(pitchValue, ELEVATOR_MIN_ADC, ELEVATOR_MAX_ADC, ELEVATOR_MIN_LOWRATES, ELEVATOR_MAX_LOWRATES) + ELEVATOR_OFFSET;
  }
  else
  {
    pitchValue = map(pitchValue, ELEVATOR_MIN_ADC, ELEVATOR_MAX_ADC, ELEVATOR_MIN, ELEVATOR_MAX) + ELEVATOR_OFFSET;
  }
  
  pitchValue = constrain(pitchValue, ELEVATOR_MIN, ELEVATOR_MAX);

  //Serial.println(pitchValue);

  return;
}



void gitThrottle()
{
  throttleValue = map(analogRead(throttlePin), THROTTLE_MIN_ADC, THROTTLE_MAX_ADC, THROTTLE_MIN, THROTTLE_MAX);
  throttleValue = constrain(throttleValue, THROTTLE_MIN, THROTTLE_MAX);

  //Serial.println(throttleValue);

  return;
}



void gitRoll()
{
  Serial.println(digitalRead(1));
  rollValue = analogRead(rollPin);

  if(digitalRead(1))
  {
    rollValue = map(rollValue, AILERON_MIN_ADC, AILERON_MAX_ADC, AILERON_MAX_LOWRATES, AILERON_MIN_LOWRATES) + AILERON_OFFSET;
  }
  else
  {
    rollValue = map(rollValue, AILERON_MIN_ADC, AILERON_MAX_ADC, AILERON_MAX, AILERON_MIN) + AILERON_OFFSET;
  }
  
  rollValue = constrain(rollValue, AILERON_MIN, AILERON_MAX);

  //Serial.println(rollValue);

  return;
}



void gitYaw()
{
  yawValue = analogRead(yawPin);

  if(digitalRead(2))
  {
    yawValue = map(yawValue, RUDDER_MIN_ADC, RUDDER_MAX_ADC, RUDDER_MAX_LOWRATES, RUDDER_MIN_LOWRATES) + RUDDER_OFFSET;
  }
  else
  {
    yawValue = map(yawValue, RUDDER_MIN_ADC, RUDDER_MAX_ADC, RUDDER_MAX, RUDDER_MIN) + RUDDER_OFFSET - 7;
  }
  
  yawValue = constrain(yawValue, RUDDER_MIN, RUDDER_MAX);

  //Serial.println(yawValue);

  return;
}












