/*/////////////////////////////////////////////////////////////////////////*/
/* needed libraries */
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BNO055.h>
#include <utility/imumaths.h>
#include <Adafruit_PWMServoDriver.h>
#include <LIDARLite.h>
#include <Servo.h>
/*/////////////////////////////////////////////////////////////////////////*/



/*/////////////////////////////////////////////////////////////////////////*/
/* macros */
#define M_PI   3.141593
#define DENSITY 1.225 /* kg/m^3 - air at sea level */

#define PITOT_PIN 0

#define THROTTLE_PIN 28
#define NOSE_GEAR_PIN 4
#define R_AILERON_PIN 3
#define L_AILERON_PIN 2
#define ELEVATOR_PIN 1
#define RUDDER_PIN 0

#define THROTTLE_MAX 170
#define AILERON_MAX 340
#define ELEVATOR_MAX 390
#define RUDDER_MAX 335

#define THROTTLE_MIN 50
#define AILERON_MIN 274
#define ELEVATOR_MIN 224
#define RUDDER_MIN 234

#define MAXPITCHUP -20
#define UNSAFEPITCHUP -35

#define MAXPITCHDOWN 10
#define UNSAFEPITCHDOWN 30

#define MAXROLL_RIGHT -35
#define UNSAFEROLL_RIGHT -45
#define MAXROLL_LEFT 35
#define UNSAFEROLL_LEFT 45

#define PITCH_CORRECTION_MIN 0
#define PITCH_CORRECTION_MAX 30

#define ROLL_CORRECTION_MIN 0
#define ROLL_CORRECTION_MAX 10

#define SERVO_MIN 0
#define SERVO_MAX 0
/*/////////////////////////////////////////////////////////////////////////*/



/*/////////////////////////////////////////////////////////////////////////*/
/* class initializations */
Adafruit_BNO055 bno = Adafruit_BNO055(55);
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(0x40);
LIDARLite myLidarLite;
Servo throttle;
/*/////////////////////////////////////////////////////////////////////////*/



/*/////////////////////////////////////////////////////////////////////////*/
/* data transfer variables */
byte maxPacketSize = 14;
byte delimiterFound = 0;

byte messageID = 0;
byte data1 = 0;
byte data2 = 0;
/*/////////////////////////////////////////////////////////////////////////*/



/*/////////////////////////////////////////////////////////////////////////*/
/* sensor variables */
byte LiDAR_Counter = 0;
int altitude = 0;
float rollAngle = 0;
float pitchAngle = 0;
float convertedRoll = 0;
float convertedPitch = 0;
int convertedAltitude = 0;
int pressure = 0; /* Pa */
float velocity = 0; /* m/s */
float latitude = 0;
float longitude = 0;
/*/////////////////////////////////////////////////////////////////////////*/



void setup()
{
  Serial.begin(115200); /* PC debugging */
  Serial4.begin(9600);  /* XBee comms   */
  Serial3.begin(9600);  /* GPS comms    */

  while(!Serial);
  Serial.println("hi");

  pinMode(13,OUTPUT);
  digitalWrite(13,HIGH);

  if(!bno.begin())
  {
    /* There was a problem detecting the BNO055 ... check your connections */
    Serial.print("Ooops, no BNO055 detected ... Check your wiring or I2C ADDR!");
    //while(1);
  }
  delay(1000);
  bno.setExtCrystalUse(true);

  pwm.begin();
  pwm.setPWMFreq(50);

  throttle.attach(THROTTLE_PIN);

  //myLidarLite.begin(0, true);
  //myLidarLite.configure(0);
}



void loop()
{
  /* read IMU data */
  //getEulerAngles();

  /* read altimeter data */
  //readLiDAR();

  /* read airspeed data */
  //readPitot();

  /* find true altitude */
  //convertedAltitude = altitude * cos(convertedRoll) * cos(convertedPitch);

  /* get GS data if available */
  grabData_Serial4();

  /* get position data if available */
  //grabData_LatLong();
}



void getEulerAngles()
{
  /* Get a new sensor event */
  sensors_event_t event;
  bno.getEvent(&event);

  rollAngle = event.orientation.y;
  pitchAngle = event.orientation.z;

  /* convert Euler angles from degrees to radians */
  convertedRoll = (rollAngle) * (M_PI / 180);
  convertedPitch = (pitchAngle) * (M_PI / 180);
}



void readLiDAR()
{
  if(LiDAR_Counter >= 100)
  {
    altitude = myLidarLite.distance();
    LiDAR_Counter = 0;
  }
  else
  {
    altitude = myLidarLite.distance(false);
    LiDAR_Counter = LiDAR_Counter + 1;
  }
}



void readPitot()
{
  pressure = map(analogRead(PITOT_PIN), 103, 920, -2000, 2000);

  velocity = sqrt(2*pressure/DENSITY); /* m/s */
  
  return;
}



void grabData_Serial4()
{
  byte trash;

  /* find delimiter */
  if ((Serial4.available()) && (delimiterFound == 0))
  {
    if (Serial4.read() == 200)
    {
      delimiterFound = 1;
    }
    else
    {
      delimiterFound = 0;
    }
  }

  /* find data if size of packet has been found */
  if ((Serial4.available() >= maxPacketSize) && (delimiterFound == 1))
  {
    while (Serial4.peek() != 100)
    {
      /* grab data */
      messageID = Serial4.read();
      data1 = Serial4.read();
      data2 = Serial4.read();

      /* analyze data grabbed from Serial */
      analyzeDATA(messageID, data1, data2);
    }
    trash = Serial4.read(); /* eat up EOF byte */

    /* reset variables for next transmission */
    delimiterFound = 0;
  }

  return;
}



void analyzeDATA(byte messageID, byte data1, byte data2)
{
  int processedData = (data1 << 8) | data2;

  if (messageID == 4) /*//////////////////Throttle*/
  {
    throttle.write(constrain(data2, THROTTLE_MIN, THROTTLE_MAX));
    Serial.println(data2);
  }
  else if (messageID == 1) /*//////////////////Pitch*/
  {
    eulerLimit(1, processedData);
    processedData = constrain(processedData, ELEVATOR_MIN, ELEVATOR_MAX);
    pwm.setPWM(ELEVATOR_PIN, 0, processedData);
  }
  else if (messageID == 2) /*//////////////////Roll*/
  {
    eulerLimit(2, processedData);
    processedData = constrain(processedData, AILERON_MIN, AILERON_MAX);
    pwm.setPWM(R_AILERON_PIN, 0, processedData);
    pwm.setPWM(L_AILERON_PIN, 0, processedData);
  }
  else if (messageID == 3) /*//////////////////Yaw*/
  {
    processedData = constrain(processedData, RUDDER_MIN, RUDDER_MAX);
    pwm.setPWM(RUDDER_PIN, 0, processedData);
    processedData = map(processedData, RUDDER_MIN, RUDDER_MAX, 410, 204)+50;
    processedData = constrain(processedData, 204, 410);
    pwm.setPWM(NOSE_GEAR_PIN, 0, processedData);
  }
  else
  {
    /* do nothing */
  }

  return;
}



void grabData_LatLong()
{
  byte fixTest;
  
  byte Letter_1 = 'G';
  byte Letter_2 = 'P';
  byte Letter_3 = 'G';
  byte Letter_4 = 'L';
  byte Letter_5 = 'L';
  
  byte Letter_1_Actual;
  byte Letter_2_Actual;
  byte Letter_3_Actual;
  byte Letter_4_Actual;
  byte Letter_5_Actual;
  
  if(Serial3.available())
  {
    while((Serial3.read() != '$') || (!Serial3.available())); /* Sync up to NMEA */
    
    while(Serial3.available() < 5); /* Wait for full NMEA identifier */
    
    Letter_1_Actual = Serial3.read();  /* Read full NMEA identifier */
    Letter_2_Actual = Serial3.read();
    Letter_3_Actual = Serial3.read();
    Letter_4_Actual = Serial3.read();
    Letter_5_Actual = Serial3.read();

    if((Letter_1_Actual == Letter_1) && (Letter_2_Actual == Letter_2) && (Letter_3_Actual == Letter_3) && (Letter_4_Actual == Letter_4) && (Letter_5_Actual == Letter_5))
    {
      while(Serial3.available() < 25); /* Wait for all lat long data (assuming only operating in N and W hemispheres) */

      Serial3.read(); /* Trash comma */

      fixTest = Serial3.read();

      if(fixTest != ',') /* Execute only if GPS has a fix */
      {
        longitude = ASCIIToNum(fixTest)*1000;
        longitude = longitude + ASCIIToNum(Serial3.read())*100;
        longitude = longitude + ASCIIToNum(Serial3.read())*10;
        longitude = longitude + ASCIIToNum(Serial3.read());
        Serial3.read(); /* Trash period */
        longitude = longitude + ASCIIToNum(Serial3.read()) / 10.0;
        longitude = longitude + ASCIIToNum(Serial3.read()) / 100.0;
        longitude = longitude + ASCIIToNum(Serial3.read()) / 1000.0;
        longitude = longitude + ASCIIToNum(Serial3.read()) / 10000.0;
        longitude = longitude + ASCIIToNum(Serial3.read()) / 100000.0;

        Serial3.read(); /* Trash comma */
        Serial3.read(); /* Trash N */
        Serial3.read(); /* Trash comma */

        latitude = ASCIIToNum(Serial3.read())*10000;
        latitude = latitude + ASCIIToNum(Serial3.read())*1000;
        latitude = latitude + ASCIIToNum(Serial3.read())*100;
        latitude = latitude + ASCIIToNum(Serial3.read())*10;
        latitude = latitude + ASCIIToNum(Serial3.read());
        Serial3.read(); /* Trash period */
        latitude = latitude + ASCIIToNum(Serial3.read()) / 10.0;
        latitude = latitude + ASCIIToNum(Serial3.read()) / 100.0;
        latitude = latitude + ASCIIToNum(Serial3.read()) / 1000.0;
        latitude = latitude + ASCIIToNum(Serial3.read()) / 10000.0;
        latitude = latitude + ASCIIToNum(Serial3.read()) / 100000.0;
      }
    }
  }
  
  return;
}



byte ASCIIToNum(byte input)
{
  byte output = 255;
  
  if((input >= '0') && (input <= '9'))
  {
    output = input - '0';
  }
  
  return output;
}



void eulerLimit(byte angleType, int &processedData)
{  
  if(angleType) ///////////////////////////pitch
  {
    if(pitchAngle < UNSAFEPITCHUP)
    {
      processedData = ELEVATOR_MAX;
    }
    else if(pitchAngle < MAXPITCHUP)
    {
      processedData = processedData + map(pitchAngle, MAXPITCHUP, UNSAFEPITCHUP, PITCH_CORRECTION_MIN, PITCH_CORRECTION_MAX);
      processedData = constrain(processedData, ELEVATOR_MIN, ELEVATOR_MAX);
    }
    else
    {
      //do nothing
    }

    

    if(pitchAngle > UNSAFEPITCHDOWN)
    {
      processedData = ELEVATOR_MIN;
    }
    else if(pitchAngle > MAXPITCHDOWN)
    {
      processedData = processedData - map(pitchAngle, MAXPITCHDOWN, UNSAFEPITCHDOWN, PITCH_CORRECTION_MIN, PITCH_CORRECTION_MAX);
      processedData = constrain(processedData, ELEVATOR_MIN, ELEVATOR_MAX);
    }
    else
    {
      //do nothing
    }
  }


  
  else ////////////////////////////////////roll
  {
    if(rollAngle < UNSAFEROLL_RIGHT)
    {
      processedData = AILERON_MAX;
    }
    else if(rollAngle < MAXROLL_RIGHT)
    {
      processedData = processedData + map(rollAngle, MAXROLL_RIGHT, UNSAFEROLL_RIGHT, ROLL_CORRECTION_MIN, ROLL_CORRECTION_MAX);
      processedData = constrain(processedData, AILERON_MIN, AILERON_MAX);
    }
    else
    {
      //do nothing
    }



    if(rollAngle < UNSAFEROLL_LEFT)
    {
      processedData = AILERON_MIN;
    }
    else if(rollAngle < MAXROLL_LEFT)
    {
      processedData = processedData - map(rollAngle, MAXROLL_LEFT, UNSAFEROLL_LEFT, ROLL_CORRECTION_MIN, ROLL_CORRECTION_MAX);
      processedData = constrain(processedData, AILERON_MIN, AILERON_MAX);
    }
    else
    {
      //do nothing
    }
  }
  
  return;
}














